# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: eeprom_mapping.proto

# pylint: skip-file
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='eeprom_mapping.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14\x65\x65prom_mapping.proto\"\xa4\x04\n\x0c\x45\x65promLayout\x12,\n\x03\x64\x61\x63\x18\x01 \x01(\x0b\x32\x1f.EepromLayout.ModuleCalibParams\x12,\n\x03\x61\x64\x63\x18\x02 \x01(\x0b\x32\x1f.EepromLayout.ModuleCalibParams\x12,\n\x03rtd\x18\x03 \x01(\x0b\x32\x1f.EepromLayout.ModuleCalibParams\x12+\n\x02tc\x18\x04 \x01(\x0b\x32\x1f.EepromLayout.ModuleCalibParams\x12(\n\nconfig_key\x18\x05 \x01(\x0b\x32\x14.EepromLayout.AwsKey\x12&\n\x08\x64\x61ta_key\x18\x06 \x01(\x0b\x32\x14.EepromLayout.AwsKey\x12\x15\n\rserial_number\x18\x07 \x01(\t\x12\r\n\x05model\x18\x08 \x01(\t\x12\x11\n\tclient_id\x18\t \x01(\t\x1a\x9d\x01\n\x11ModuleCalibParams\x12<\n\x06\x63\x61libs\x18\x01 \x03(\x0b\x32,.EepromLayout.ModuleCalibParams.ChannelCalib\x1aJ\n\x0c\x43hannelCalib\x12\x11\n\x04gain\x18\x01 \x01(\x02H\x00\x88\x01\x01\x12\x13\n\x06offset\x18\x02 \x01(\x02H\x01\x88\x01\x01\x42\x07\n\x05_gainB\t\n\x07_offset\x1a\x32\n\x06\x41wsKey\x12\x13\n\x0bprivate_key\x18\x01 \x01(\t\x12\x13\n\x0b\x63\x65rtificate\x18\x02 \x01(\tb\x06proto3'
)




_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB = _descriptor.Descriptor(
  name='ChannelCalib',
  full_name='EepromLayout.ModuleCalibParams.ChannelCalib',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='gain', full_name='EepromLayout.ModuleCalibParams.ChannelCalib.gain', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='EepromLayout.ModuleCalibParams.ChannelCalib.offset', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='_gain', full_name='EepromLayout.ModuleCalibParams.ChannelCalib._gain',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_offset', full_name='EepromLayout.ModuleCalibParams.ChannelCalib._offset',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=447,
  serialized_end=521,
)

_EEPROMLAYOUT_MODULECALIBPARAMS = _descriptor.Descriptor(
  name='ModuleCalibParams',
  full_name='EepromLayout.ModuleCalibParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='calibs', full_name='EepromLayout.ModuleCalibParams.calibs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=364,
  serialized_end=521,
)

_EEPROMLAYOUT_AWSKEY = _descriptor.Descriptor(
  name='AwsKey',
  full_name='EepromLayout.AwsKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='private_key', full_name='EepromLayout.AwsKey.private_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='certificate', full_name='EepromLayout.AwsKey.certificate', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=523,
  serialized_end=573,
)

_EEPROMLAYOUT = _descriptor.Descriptor(
  name='EepromLayout',
  full_name='EepromLayout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dac', full_name='EepromLayout.dac', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adc', full_name='EepromLayout.adc', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rtd', full_name='EepromLayout.rtd', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tc', full_name='EepromLayout.tc', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='config_key', full_name='EepromLayout.config_key', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_key', full_name='EepromLayout.data_key', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serial_number', full_name='EepromLayout.serial_number', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model', full_name='EepromLayout.model', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_id', full_name='EepromLayout.client_id', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EEPROMLAYOUT_MODULECALIBPARAMS, _EEPROMLAYOUT_AWSKEY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=573,
)

_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.containing_type = _EEPROMLAYOUT_MODULECALIBPARAMS
_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.oneofs_by_name['_gain'].fields.append(
  _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.fields_by_name['gain'])
_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.fields_by_name['gain'].containing_oneof = _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.oneofs_by_name['_gain']
_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.oneofs_by_name['_offset'].fields.append(
  _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.fields_by_name['offset'])
_EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.fields_by_name['offset'].containing_oneof = _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB.oneofs_by_name['_offset']
_EEPROMLAYOUT_MODULECALIBPARAMS.fields_by_name['calibs'].message_type = _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB
_EEPROMLAYOUT_MODULECALIBPARAMS.containing_type = _EEPROMLAYOUT
_EEPROMLAYOUT_AWSKEY.containing_type = _EEPROMLAYOUT
_EEPROMLAYOUT.fields_by_name['dac'].message_type = _EEPROMLAYOUT_MODULECALIBPARAMS
_EEPROMLAYOUT.fields_by_name['adc'].message_type = _EEPROMLAYOUT_MODULECALIBPARAMS
_EEPROMLAYOUT.fields_by_name['rtd'].message_type = _EEPROMLAYOUT_MODULECALIBPARAMS
_EEPROMLAYOUT.fields_by_name['tc'].message_type = _EEPROMLAYOUT_MODULECALIBPARAMS
_EEPROMLAYOUT.fields_by_name['config_key'].message_type = _EEPROMLAYOUT_AWSKEY
_EEPROMLAYOUT.fields_by_name['data_key'].message_type = _EEPROMLAYOUT_AWSKEY
DESCRIPTOR.message_types_by_name['EepromLayout'] = _EEPROMLAYOUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EepromLayout = _reflection.GeneratedProtocolMessageType('EepromLayout', (_message.Message,), {

  'ModuleCalibParams' : _reflection.GeneratedProtocolMessageType('ModuleCalibParams', (_message.Message,), {

    'ChannelCalib' : _reflection.GeneratedProtocolMessageType('ChannelCalib', (_message.Message,), {
      'DESCRIPTOR' : _EEPROMLAYOUT_MODULECALIBPARAMS_CHANNELCALIB,
      '__module__' : 'eeprom_mapping_pb2'
      # @@protoc_insertion_point(class_scope:EepromLayout.ModuleCalibParams.ChannelCalib)
      })
    ,
    'DESCRIPTOR' : _EEPROMLAYOUT_MODULECALIBPARAMS,
    '__module__' : 'eeprom_mapping_pb2'
    # @@protoc_insertion_point(class_scope:EepromLayout.ModuleCalibParams)
    })
  ,

  'AwsKey' : _reflection.GeneratedProtocolMessageType('AwsKey', (_message.Message,), {
    'DESCRIPTOR' : _EEPROMLAYOUT_AWSKEY,
    '__module__' : 'eeprom_mapping_pb2'
    # @@protoc_insertion_point(class_scope:EepromLayout.AwsKey)
    })
  ,
  'DESCRIPTOR' : _EEPROMLAYOUT,
  '__module__' : 'eeprom_mapping_pb2'
  # @@protoc_insertion_point(class_scope:EepromLayout)
  })
_sym_db.RegisterMessage(EepromLayout)
_sym_db.RegisterMessage(EepromLayout.ModuleCalibParams)
_sym_db.RegisterMessage(EepromLayout.ModuleCalibParams.ChannelCalib)
_sym_db.RegisterMessage(EepromLayout.AwsKey)


# @@protoc_insertion_point(module_scope)
